library memoized;

import 'package:to_string_pretty/to_string_pretty.dart';
import 'typedef.dart' show Arg0Function;

class Memoized<V> {
  final Arg0Function<V> _computation;
  bool _updateRequested = true;
  V? _memoized;

  Memoized(this._computation);

  V call({bool shouldUpdate = false}) {
    if (shouldUpdate || _updateRequested) {
      update();
      _updateRequested = false;
    }
    return _memoized!;
  }

  void update() => _memoized = _computation();

  void requestUpdate() => _updateRequested = true;

  @override
  String toString() => toStringPretty(this, {
        'functionType': '$V Function()',
        'lastComputation': _memoized?.toString() ?? 'not computed yet',
        'updateRequested': '$_updateRequested',
      });
}

extension Memo0Ext<V> on Arg0Function<V> {
  Memoized<V> get memo => Memoized(this);
}
